// we are including all the PM header files here.  Please replace with you
// stock "Include all the OS/2 PM stuff" include file
#define INCL_WIN
#define INCL_DOS
#define INCL_DOSDEVIOCTL

#define INCL_RXFUNC
#define INCL_RXARI

extern "C"
{
#include <os2.h>
#include <rexxsaa.h>
}

// include the Mesa AddIn header file
#include "extaddin.h"
#include <string.h>
#include <stdio.h>                 // added for printf
#include <stdlib.h>
#include <math.h>

// to avoid the C++ name mangling, let's define the names of the external
// function here

extern "C"
{
    void _System AddInInit(void *);
    void _System AddInFree(void *);
}

#include "sampadin.h"
#include "model.h"
#include "functions.h"
#include "range.h"
#include "addin.h"
#include "samprexx.h"

#include "solver.h"
#include "data.h"


// a couple of global values
AddIn *thisAddIn = NULL;
int solverMenuId = 0;
int dateMenuId = 0;
int dataMenuId = 0;
int which =0;

int solvCount = 0;
HWND *theSolverHwnds = NULL;

int dataCount = 0;
HWND *theDataHwnds = NULL;



// this routine is called by Mesa when it needs to either check to see if a menu
// item should be enabled or disabled or if if the command should be executed
// A command may be passed to the call-back function even if the command is not
// recognized by the call-back.  Ignore commands that your AddIn does not know
// about and return a 0 for all commands that you do not recognize
int _System  myMenuCallBackRoutine(int command,int function)
{
    int ret = 0; // the default return value
    HWND hwnd;

    switch (function) {
    // execute the command
    case 0:
        if (command == solverMenuId) {
            int x = 0;
            Model mo(-1);
            int found = 0;
            // check to see if the model already has a solver open for it
            for (x = 0;x < solvCount ; x++) {
                if (theSolverHwnds[x] && (*(Model *)WinSendMsg(theSolverHwnds[x],WM_GETMODELPTR,0,0) ==  mo)) {
                    found = 1;
                    hwnd = theSolverHwnds[x];
                } /* endif */
            } /* endfor */

            if (!found) {
                hwnd = WinLoadDlg(HWND_DESKTOP,thisAddIn-> getActiveWindow(),
                                  SolvDlgProc,(HMODULE)thisAddIn->getHandle(),ID_DLG_GOALSEEK,0L);
            } else {
                WinSetFocus(HWND_DESKTOP,hwnd);
            } /* endif */
            ret = 1;
        } else if (command == dataMenuId) {
            int x = 0;
            Model mo(-1);
            int found = 0;
            // check to see if the model already has a data filter open for it
            for (x = 0;x < dataCount ; x++) {
                if (theDataHwnds[x] && (*(Model *)WinSendMsg(theDataHwnds[x],WM_GETMODELPTR,0,0) ==  mo)) {
                    found = 1;
                    hwnd = theDataHwnds[x];
                } /* endif */
            } /* endfor */

            if (!found) {
                hwnd = WinLoadDlg(HWND_DESKTOP,thisAddIn-> getActiveWindow(),
                                  DataDlgProc,(HMODULE)thisAddIn->getHandle(),
                                  ID_DLG_DATAFILTER,0L);
            } else {
                WinShowWindow(hwnd,TRUE);
                WinSetFocus(HWND_DESKTOP,hwnd);
            } /* endif */
            ret = 1;
        } else if (command == dateMenuId) {
            // get the system date and stuff it into the current cell
            Address ad = thisAddIn -> getCurrentAddress();
            Model mo(-1);
            DATETIME dt;
            DosGetDateTime(&dt);
            char buf[100];
            sprintf(buf,"%d/%d/%d",dt.month,dt.day,dt.year);
            mo.setCellString(ad,buf);
            ret = 1;
        } // endif
        break;

        // return 1 if the menu should be enabled or a 0 if this command
        // is disabled or
        // if we do not recognize this command.  All other values are
        // reserved.
    case 1:
        // only activate when there is a window
        if ((command == solverMenuId) && thisAddIn -> getActiveWindow())
            ret = 1;
        else if (command == dataMenuId  && thisAddIn -> getActiveWindow())
            ret = 1;
        else if (command == dateMenuId  && thisAddIn -> getActiveWindow())
            ret = 1;
        break;
        // open the help screen
    case 2:
        break;
    }

    return ret;
}

// this function will be called whenever a model is closed
int _System aboutToClose(void *model_handle)
{
    int x = 0;
    Model mo((int)model_handle);
    HWND hwnd;


    // check to see if the model already has a solver/data filter open for it
    for (x = 0;x < solvCount ; x++) {
        if (theSolverHwnds[x] && (*(Model *)WinSendMsg(theSolverHwnds[x],WM_GETMODELPTR,0,0) ==  mo)) {
            WinDismissDlg(theSolverHwnds[x],0);  // if so, close it
            theSolverHwnds[x] = NULL;
        } /* endif */
    } /* endfor */
    for (x = 0;x < dataCount ; x++) {
        if (theDataHwnds[x] && (*(Model *)WinSendMsg(theDataHwnds[x],WM_GETMODELPTR,0,0) ==  mo)) {
            WinDismissDlg(theDataHwnds[x],0);  // if so, close it
            theDataHwnds[x] = NULL;
        } /* endif */
    } /* endfor */

    return 0;
}

void _System initMenuCallback(int menu, HWND menuHwnd)
{
    if (menu == 0) {
        // Add the Goal seeker to the menu
        solverMenuId = thisAddIn -> addMenuItem("~Goal Seeker...",menuHwnd);
        // Add the data filter menu item
        dataMenuId = thisAddIn -> addMenuItem("~Data Filter...",menuHwnd);

        // handle the date stuff by createing a menu ID for it and doing "magic"
        // with the accelerators
        dateMenuId = thisAddIn -> addMenuItem("~Insert Date\tCtrl+Alt+;",menuHwnd);
    } // endif
}


// initialize everything needed by this addin
void _System AddInInit(void *handle)
{
    // create an AddIn object from the handle
    thisAddIn = new AddIn((int)handle);


    if (thisAddIn -> version() < 349) {
        initMenuCallback(0,thisAddIn -> getMenu());
    } else {
        // add the menu init callback
        thisAddIn -> registerInitMenuFunction(initMenuCallback);
    }

    // get all of the data for the old accelerator table
    HWND frame = thisAddIn -> getApplicationWindow();
    HACCEL oldaccel = WinQueryAccelTable(0,frame);
    int size = WinCopyAccelTable(oldaccel,NULL,0);
    PACCELTABLE table = (PACCELTABLE)MMalloc(size+sizeof(ACCEL));
    WinCopyAccelTable(oldaccel,table,size);

    table->aaccel[table->cAccel].fs = 0x31;
    table->aaccel[table->cAccel].key = ';';
    table->aaccel[table->cAccel].cmd = dateMenuId;
    table->cAccel++;

    // create a new accelerator table and replace the old one
    HACCEL hac = WinCreateAccelTable(0,table);
    MFree(table);
    WinSetAccelTable(0,hac,frame);
    WinDestroyAccelTable(oldaccel);

    // Register a menu call back so we can get the commands when pressed
    thisAddIn -> registerMenuCallBackFunction(myMenuCallBackRoutine);

    // space to hold the solver and data filter dialog window handles
    solvCount = 1;
    theSolverHwnds = (HWND *)MMalloc(sizeof(HWND)*solvCount);
    theSolverHwnds[0]=NULL;
    dataCount = 1;
    theDataHwnds = (HWND *)MMalloc(sizeof(HWND)*dataCount);
    theDataHwnds[0]=NULL;

    // When workbooks are closed we want to close the solver and 
    // data filter window
    thisAddIn -> registerAboutToCloseFunction(aboutToClose);


    int sections[] = {14,0};

    // Register some AddIn functions
    thisAddIn -> registerFunction("NUMIN(","NUMIN","NUMIN(ITEM,RANGE)",
                                  "Counts the number of times ITEM appears in the range",
                                  "NUMIN(12,A1:C25)",sections,numIn);
    thisAddIn -> registerFunction("LAYERNAME(","LAYERNAME","LAYERNAME(LAYER)",
                                  "Returns the string representation of the layer",
                                  "LAYERNAME(0) -> A",
                                  sections,layername);
    thisAddIn -> registerFunction("BEEP(","BEEP","BEEP(FREQUENCY,DURATION)",
                                  "Beeps at the specified frequency (hz) for the specified duration (milliseconds)",
                                  "BEEP(1440,100)",
                                  sections,BeepFunction);
    thisAddIn -> registerFunction("FACTORIAL(","FACTORIAL","FACTORIAL(INTEGER)",
                                  "COMPUTES n!","FACTORIAL(4) -> 96",sections,FactorialFunc);
    thisAddIn -> registerFunction("ENGINEERING(","ENGINEERING","ENGINEERING(NUMBER)",
                                  "Returns the number in engineering notation","ENGINEERING(12345.6) -> 12.345e3",
                                  sections,EngineeringFunction);

    // register our REXX functions/extensions
    RexxRegisterFunctionExe((PSZ)"WAITFORINPUT", (PFN) waitForIntputRexxFunc);
    RexxRegisterFunctionExe((PSZ)"DATAFILTER", (PFN) dataFilterRexxFunc);
}


// we need to free the hwnd list because all of those windows are now closed
void _System AddInFree(void *)
{
    MFree(theSolverHwnds);
    MFree(theDataHwnds);
    delete thisAddIn;
}


